/*
 * Copyright (C) 2000-2005
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 
 */
 
package estella.lib;
import  estella.lib.*;
import  estella.gui.*;

import  java.io.File;
import  java.lang.String;
import  java.util.Vector;

/**
* Gerencia repositorio de arquivos
* @author Cesar Kallas
* @since 05/12/2005
*/
public class Repositorio
{
	private static String caminho;
	private Arquivo arquivo;
	private Log LOG;
	private Constantes CON;
		
	public Repositorio()
	{
		caminho = CON.REPOSITORIO;
		LOG = new Log();
	}
	
	/**
	* Gera repositorio
	*/
	public boolean gera()
	{
		try {
			synchronized(JanelaPrincipal.linhaTabelaArquivosCompartilhados) {
				JanelaPrincipal.linhaTabelaArquivosCompartilhados.clear();
			}
			
			File repositorio = new File(caminho);
			if(repositorio.isDirectory()) {
				return geraRepositorio(new File(caminho));
			}
			else
				return false;
		}
		catch(Exception SemRepositorio) {
			LOG.falha(SemRepositorio.getMessage());
			return false;
		}
	}
	
	/**
	* Lista todos arquivos (e arquivos dentro de subdiretorios) de um caminho passado
	* @param caminho diretorio inicial para busca de arquivos
	* @param Lista de arquivos definida na classe dados (Dados.arquivos)
	* @return true ou false
	* @see Dados
	*/
	private boolean geraRepositorio(File dir)
	{
		String[] arquivos = dir.list();
		int tamanho=0, numArqComp=0;
		synchronized(JanelaPrincipal.linhaTabelaArquivosCompartilhados) {
			JanelaPrincipal.linhaTabelaArquivosCompartilhados.clear();
		}
		synchronized(JanelaPrincipal.tabelaArquivosCompartilhados) {
			JanelaPrincipal.tabelaArquivosCompartilhados.removeNotify();
			JanelaPrincipal.tabelaArquivosCompartilhados.addNotify();
		}
			
		try {
			for (int i=0; i <arquivos.length; i++) {
				
				/** 
				* Se for um diretorio, entre nele e tente listar os arquivos (recursivamente)
				* Se nao, adiciona esse arquivo na lista
				*/
				File arquivoAtual = new File(dir, arquivos[i]);
				if (arquivoAtual.isDirectory())
					geraRepositorio(arquivoAtual);
				else {
				// Encontrado um novo arquivo, adiciona ele na lista de arquivos
					Vector tabelaArquivos = new Vector();
					arquivo = new Arquivo();
					arquivo.nome = arquivos[i];
					arquivo.tamanho = (int)arquivoAtual.length();
					arquivo.index = System.identityHashCode(arquivoAtual);
					arquivo.caminho = dir.getName();
					tabelaArquivos.addElement(arquivo.nome);
					tabelaArquivos.addElement(arquivo.tamanho + "");
					tabelaArquivos.addElement(arquivo.index + "");
					synchronized(JanelaPrincipal.linhaTabelaArquivosCompartilhados) {
						JanelaPrincipal.linhaTabelaArquivosCompartilhados.addElement(tabelaArquivos);
					}	
					synchronized(Dados.arquivos) {
						Dados.arquivos.adiciona(arquivo);
					}
					numArqComp++;
					tamanho+= arquivo.tamanho;
					//LOG.info( arquivo.caminho + "\\" + arquivo.nome + "\t" + arquivo.tamanho + " index: " + arquivo.index);
				}
			}
			// Adicona resultado na tabela de arquivos compartilhados da interface grafica
			synchronized(JanelaPrincipal.tabelaArquivosCompartilhados) {
				JanelaPrincipal.tabelaArquivosCompartilhados.removeNotify();
				JanelaPrincipal.tabelaArquivosCompartilhados.addNotify();
			}
			Dados.tamArqComp = tamanho;
			Dados.numArqComp = numArqComp;

			synchronized(JanelaPrincipal.labelArqComp) {
				JanelaPrincipal.labelArqComp.setText(Dados.numArqComp + " / " + (Dados.tamArqComp/1024) + " (KB)");
			}
			
			return true;
		}
		catch(Exception FalhaListarArquivos) {
			LOG.falha(FalhaListarArquivos.getMessage());
			return false;
		}
	}
}
